/**
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is kelis.fr code.
 *
 * The Initial Developer of the Original Code is 
 * sylvain.spinelli@kelis.fr
 *
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * samuel.monsarrat@kelis.fr
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
 * instead of those above. If you wish to allow use of your version of this file
 * only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
 * others to use your version of this file under the terms of the MPL, indicate
 * your decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
 * delete the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */

/**
 * SCENARI Core API.
 */
var scCoLib = {
	fDebug : false,
	fOnLoadDone:false, 
	fOnUnloadDone:false
}

// Tableau d'objets à activer sur onLoad:
//	- field "loadSortKey" optionnel (placé en fin si null)
//	- méthode "onLoad" obligatoire
var scOnLoads = [];

// Tableau d'objets à activer sur onUnload :
//	- field "unloadSortKey" optionnel (placé en fin si null)
//	- méthode "onUnload" obligatoire
var scOnUnloads = [];

/**
 * Tri et exécute les méthodes onLoad() de tous les objets dans scOnLoads[]
 */
function scOnLoad() {
	scOnLoads.sort(function (p1, p2){
			if(!p1.loadSortKey) return p2.loadSortKey ? -1 : 0;
			if(scCoLib.isIE) return p1.loadSortKey.localeCompare(p2.loadSortKey||"");
			try{
				return p1.loadSortKey > p2.loadSortKey||"" ? 1 : p1.loadSortKey == p2.loadSortKey ? 0 : -1;
			}catch(e){
				return p1.loadSortKey.localeCompare(p2.loadSortKey||"");
			}
		}
	);	for (var i in scOnLoads) try{scOnLoads[i].onLoad();}catch(e){}
	scCoLib.fOnLoadDone = true;
}

/**
 * Tri et exécute les méthodes onUnload() de tous les objets dans scOnUnloads[]
 */
function scOnUnload() {
	scOnUnloads.sort(function (p1, p2){
			if(!p1.unloadSortKey) return p2.unloadSortKey ? -1 : 0;
			if(scCoLib.isIE) return p1.unloadSortKey.localeCompare(p2.unloadSortKey||"");
			try{
				return p1.unloadSortKey > p2.unloadSortKey||"" ? 1 : p1.unloadSortKey == p2.unloadSortKey ? 0 : -1;
			}catch(e){
				return p1.unloadSortKey.localeCompare(p2.unloadSortKey||"");
			}
		}
	);
	for (var i in scOnUnloads) try{scOnUnloads[i].onUnload();}catch(e){}
	scCoLib.fOnUnloadDone = true;
}

//Tableau d'objets :
//	- field "resizeSortKey" optionnel (placé en fin si null)
//	- méthode "onResize" obligatoire
var scOnResizes = [];

/**
 * Tri et exécute les méthodes onResize() de tous les objets dans scOnResizes[]
 */
function scOnResize(pEvent) {
	var vLen = scOnResizes.length;
	if(vLen > 1 && vLen != window.onresize.lastLen) {
		scOnResizes.sort(function (p1, p2){
				if(!p1.resizeSortKey) return p2.resizeSortKey ? -1 : 0;
				if(scCoLib.isIE) return p1.resizeSortKey.localeCompare(p2.resizeSortKey||"");
				try{
					return p1.resizeSortKey > p2.resizeSortKey||"" ? 1 : p1.resizeSortKey == p2.resizeSortKey ? 0 : -1;
				}catch(e){
					return p1.resizeSortKey.localeCompare(p2.resizeSortKey||"");
				}
			}
		);
		window.onresize.lastLen = vLen;
	}
	for (var i =0; i < vLen; i++) try{scOnResizes[i].onResize(pEvent);}catch(e){}
}

/* Abonnements aux events du document. */
window.onload = scOnLoad;
window.onunload = scOnUnload;
window.onresize = scOnResize;

/* Alias à document.getElementById() */
function $(pId) { return document.getElementById(pId);}

/**
 * Ajoute pHanlder à scOnLoads[] si scOnLoad() n'a pas été appellé, sinon exécute directement pHanlder.onLoad()
 */
scCoLib.addOnLoadHandler = function(pHanlder){
	if(scCoLib.fOnLoadDone) try{pHanlder.onLoad();}catch(e){}
	else scOnLoads[scOnLoads.length] = pHanlder;
}

/**
 * Ajoute pHanlder à scOnUnloads[] si scOnUnload() n'a pas été appellé, sinon exécute directement pHanlder.onUnload()
 */
scCoLib.addOnUnloadHandler = function(pHanlder){
	if(scCoLib.fOnUnloadDone) try{pHanlder.onUnload();}catch(e){}
	else scOnUnloads[scOnUnloads.length] = pHanlder;
}


/** Gestion des navigateurs. */
scCoLib.userAgent = navigator.userAgent.toLowerCase();
scCoLib.isIE = scCoLib.userAgent.indexOf("msie")!=-1;

/**
 * Fonction wrapper de parseInt qui retourne 0 si parseInt retourne NaN (c.a.d. pX ne commençant pas par un chiffre)
 */
scCoLib.toInt = function(pX){
	var vY;
	return isNaN(vY = parseInt(pX))? 0 : vY;
}

/**
 * =============================================================================
 * utilities
 */
scCoLib.util = {
	fUndef : "undefined",
	logError : function(pPre, pEx) {
		var vMsg = pPre + ((pEx != null) ? " - "+((typeof pEx.message != "undefined") ? pEx.message : pEx) : "");
		if(window.console) {
			window.console.log(vMsg);
		} else if (scCoLib.fScConsole){
			var vMsgDiv = document.createElement("div");
			vMsgDiv.innerHTML = vMsg;
			scCoLib.fScConsole.appendChild(vMsgDiv);
		} else {
			alert(vMsg);
		}
	},
	log : function(pMsg) {
		if (!scCoLib.fDebug) return;
		if(window.console) {
			window.console.log(pMsg);
		} else if (scCoLib.fScConsole){
			var vMsgDiv = document.createElement("div");
			vMsgDiv.innerHTML = pMsg;
			scCoLib.fScConsole.appendChild(vMsgDiv);
		}
	}
}
