/**
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is kelis.fr code.
 *
 * The Initial Developer of the Original Code is 
 * samuel.monsarrat@kelis.fr
 *
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
 * instead of those above. If you wish to allow use of your version of this file
 * only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
 * others to use your version of this file under the terms of the MPL, indicate
 * your decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
 * delete the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */

/** 
 * Librairie de scripts utilitaires pour UI dynamique (collapsableBlocks etc) 
 */ 
var scDynUiMgr = {
	
	// BEGIN INTERNAL functions
/**
 * xInitMgr : init de la librairie
 */
	xInitMgr: function() {
	},

	// BEGIN EXTERNAL functions
/**
 * collBlkToggle : permet de "collapser"/"décollapser" un contenu
 * @params:  pTitle : Element titre du CollapsableBlock (anchor)
 *           pCo : Element contenu à collapser
 *           pClassOpen : classe CSS à affecter à pTitle quand pCo est visible
 *           pClassClosed : classe CSS à affecter à pTitle quand pCo est invisible
 * @returns: true if new state open, false if closed and null on error.
 */
	collBlkToggle: function(pTitle, pCo, pClassOpen, pClassClosed) {
		if (pTitle.nodeType != 1 || pCo.nodeType != 1){
			scCoLib.util.log("scDynUiMgr.collBlkToggle error : pTitle or pCo not an element.");
			return null;
		}
		vOldStateClosed = pCo.style.display == 'none';
		pCo.style.display = vOldStateClosed ? '' : 'none';
		pTitle.className = vOldStateClosed ? pClassOpen : pClassClosed;
		if (scCoLib.isIE) {
			var vTags = pCo.getElementsByTagName("IFRAME");
			if (vOldStateClosed) {
				for(var i=0; i<vTags.length; i++) {
					vTags[i].src = vTags[i].bkpSrc ? vTags[i].bkpSrc : vTags[i].src;
				}		
			} else {
				for(var i=0; i<vTags.length; i++) {
					vTags[i].bkpSrc = vTags[i].src;
					vTags[i].src = "";
				}
			}
		}
		return vOldStateClosed;
		if("scSiLib" in window) scSiLib.fireResizedNode(pCo);
	},
	
/**
 * displaySubWindow : permet de charger une URL dans une iFrame 'modale' au dessus du contenu
 * @params: pNode : Element ancre appelant displaySubWindow
 *          pUrl : Url à charger dans l'iframe modale
 *          pName : nom du subWindow, utilisé pour le stylage etc.
 *          pOpt : Options d'affichage
 */
	displaySubWindow: function(pNode,pUrl,pName,pOpt) {
		scCoLib.util.log("scDynUiMgr.displaySubWindow("+pUrl+","+pName+","+pOpt+")");
		var vOpt = pOpt || ""; //Retreave  display Opts if any...
		var vSwId = pNode.swId || null; //Retreave the subWindow ID if it has already been created
		//If the subWindow has not been created, create it !
		if (vSwId == null) { 
			vSwId = this.subWindow.xGenId("scSubWindow"); //generate a new subWindow ID
			var vSwHtml = scDynUiMgr.subWindow.xMakeSw(vSwId, pName, vOpt); //build the subWindow HTML
			var vTmpDiv=document.createElement("DIV"); // Temp div to hold the created subWindow html
			vTmpDiv.innerHTML = vSwHtml;
			var vSwOver = vTmpDiv.firstChild;
			while(vSwOver && vSwOver.nodeType != 1) vSwOver = vSwOver.nextSibling;
			var vSwDiv = vSwOver;
			vSwDiv = vSwDiv.nextSibling;
			while(vSwDiv && vSwDiv.nodeType != 1) vSwDiv = vSwDiv.nextSibling;
			var vCont = vOpt.ANCHORPATH ? scPaLib.findNode(vOpt.ANCHORPATH, pNode) || document.body : document.body;
			vCont.appendChild(vSwOver); //Apend the created subWindow overlay to the end of the document
			vCont.appendChild(vSwDiv); //Apend the created subWindow to the end of the document
			vSwDiv.fNode = pNode; //Keep pointer to owner node on the subWindow
			pNode.swId = vSwId; 
			scDynUiMgr.subWindow.xInit(vSwId);
		}
		//show subWindow
		scDynUiMgr.subWindow.xShow(vSwId,pUrl);
		//notify listeners
		for (i in scDynUiMgr.subWindow.fOpenListeners) try{scDynUiMgr.subWindow.fOpenListeners[i](vSwId);}catch(e){}
	},
/**
 * hideSubWindow : permet de cacher une subWindow
 * @params: pId : Id du subWindow à cacher
 */
	hideSubWindow: function(pId) {
		scCoLib.util.log("scDynUiMgr.hideSubWindow("+pId+")");
		var vCurrSw = scDynUiMgr.subWindow.fSubWins[pId];
		if (vCurrSw){
			vCurrSw.fOver.style.visibility = "hidden";
			vCurrSw.style.visibility = "hidden";
			vCurrSw.fFra.src = "";
			//notify listeners
			for (i in scDynUiMgr.subWindow.fCloseListeners) try{scDynUiMgr.subWindow.fCloseListeners[i](pId);}catch(e){}
		}
	}
}

/**
 * =============================================================================
 * subWindow
 */
scDynUiMgr.subWindow = {
	fZIndex : 1000,
	fSubWins : new Array(),
	fOpenListeners : new Array(),
	fCloseListeners : new Array(),
	
	addOpenListener: function(pFunc) {this.fOpenListeners.push(pFunc)},
	addCloseListener: function(pFunc) {this.fCloseListeners.push(pFunc)},
	xBuildCls: function(pCls, pSufx) {
		var vCls = pCls.split(" ");
		var vRetCls = "";
		for(var i=0; i<vCls.length; i++) vRetCls += vCls[i]+(pSufx ? ('_'+pSufx) : '')+' ';
		return(vRetCls);
	},
	xGenId: function(pPrefix) {
		var vIndex = 0;
		while(($(pPrefix+vIndex) || $(pPrefix+vIndex+'fr')) && vIndex < 10000) vIndex++;
		if (vIndex == 10000) {
			scCoLib.util.logError("scDynUiMgr ERROR: Error generating ID, too many "+pPrefix+" IDs on this page.");
			return("");
		} else return (pPrefix + vIndex);
	},
	xMakeSw: function(pId,pName,pOpt) {
		scCoLib.util.log("scDynUiMgr.subWindow.xMakeSw("+pId+","+pName+")");
		var vSubWinTi = pOpt.SUBWINTI || ""
		var vCloseBtnCo = pOpt.CLOSEBTNCO || "X"
		var vCloseBtnTi = pOpt.CLOSEBTNTI || ""
		var vHtml = '<div id="'+pId+'_over" class="'+this.xBuildCls(pName,"over");
		vHtml += '" style="position:absolute;visibility:hidden;"></div>';
		vHtml += '<div id="'+pId+'" class="'+this.xBuildCls(pName,"win")+'" style="position:absolute;visibility:hidden;">';
		vHtml += '<div class="'+this.xBuildCls(pName,"ti")+'">';
		vHtml += '<a class="'+this.xBuildCls(pName,"x")+'"'+(vCloseBtnTi ? ' title="'+vCloseBtnTi+'"' : '')+' href="#" onclick="scDynUiMgr.hideSubWindow(\''+pId+'\');return(false);"><span>'+vCloseBtnCo+'</span></a>';
		vHtml += '<span id="'+pId+'_ti">'+vSubWinTi+'</span>';
		vHtml += '</div>';
		vHtml += '<div class="'+this.xBuildCls(pName,"co")+'">';
		vHtml += '<iframe class="'+this.xBuildCls(pName,"fra")+'" marginheight="0" marginwidth="0" frameborder="0';
		vHtml += '" name="'+pName+'" src=""></iframe>';
		return(vHtml+'</div></div>');
	},
	
	xInit: function(pId) {
		scCoLib.util.log("scDynUiMgr.subWindow.xInit("+pId+")");
		var vCurrSw = $(pId);
		vCurrSw.fOver = $(pId+"_over");
		vCurrSw.fTi = $(pId+"_ti");
		vCurrSw.fFra = vCurrSw.getElementsByTagName("IFRAME")[0];
		vCurrSw.fFra.setSubWindowTitle = function(pTitle){
			scDynUiMgr.subWindow.fSubWins[pId].fTi.innerHTML = pTitle;
		}
		vCurrSw.fFra.hideSubWindow = function(){
			scDynUiMgr.hideSubWindow(pId);
		}
		this.fSubWins[pId] = vCurrSw;
	},
	
	xShow: function(pId,pUrl) {
		scCoLib.util.log("scDynUiMgr.subWindow.xShow("+pId+","+pUrl+")");
		var vCurrSw = this.fSubWins[pId];
		vCurrSw.fFra.src = pUrl;
		vCurrSw.fOver.style.zIndex = this.fZIndex++;
		vCurrSw.fOver.style.visibility = "visible";
		vCurrSw.style.zIndex = this.fZIndex++;
		vCurrSw.style.visibility = "visible";
	}
}

/**
 * =============================================================================
 * init librairie et abonnements events
 */
scDynUiMgr.xInitMgr();
