/**
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is kelis.fr code.
 *
 * The Initial Developer of the Original Code is 
 * sylvain.spinelli@kelis.fr
 *
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * samuel.monsarrat@kelis.fr
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
 * instead of those above. If you wish to allow use of your version of this file
 * only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
 * others to use your version of this file under the terms of the MPL, indicate
 * your decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
 * delete the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */

/**
 * SCENARI Path API.
 */
var scPaLib = {};

/**
 * Recherche un noeud.
 * Syntaxe : 
 * axis:filter[/axis:filter]
 * 
 * Axis : 
 *  anc: ancestors
 *  par: parent
 *  chi: children
 *  chl: children from last to first
 *  nsi: next siblings
 *  psi: previous siblings
 *  cde: current or descendants
 *  des: descendants
 *  bod: body du document
 *  ide:yyy recherche de l'elt d'id yyy.
 *
 * Filters  : voir scPaLib.checkNode(pFilter, pNode)
 */
scPaLib.findNode= function(pPath, pFrom) {
	return this.xFindNode(pPath.scPath ? pPath : this.compilePath(pPath), pFrom || document, 0);
}


/**
 * Recherche une liste noeud et retourne un tableau JS de noeuds correspondant au path.
 */
scPaLib.findNodes = function(pPath, pFrom) {
	var vResult = [];
	this.xFindNodes(pPath.scPath ? pPath : this.compilePath(pPath), pFrom || document, 0, vResult);
	return vResult;
}

/**
 * Controle un noeud par rapport à un filtre.
 * 
 * Filters  : 
 * - "" : tout noeud
 * - "xxx" : élément nommé xxx
 * - ".yy" : élément contenant la classe yy
 * - "xxx.yy.zz" : élément nommé xxx contenant les classes yy ET zz
 * - "h\d" : match une balise de type h1, h2, h3, etc.
 * - "aaa|bbb.yy|.zz" : élément nommé aaa ou élément nommé bbb contenant la classe yy ou élément contenant la classe zz
 *
 */
scPaLib.checkNode = function(pFilter, pNode){
	var vFilter = pFilter.scFilter ? pFilter : this.compileFilter(pFilter);
	if (vFilter.scFilter == 1) return this.xCheckNode(vFilter,pNode);
	else {
		var vMatch = false;
		for(var i = 0; i < vFilter.length; i++) {
			if (vMatch = this.xCheckNode(vFilter[i],pNode)) break; 
		}	
		return vMatch;
	}
}

/**
 * Cherche un mot entier dans une liste de mots séparés par des virgules, des espaces, des points ou des tirets.
 */
scPaLib.containWord = function(pString, pWord){
	var vIdx = pString.indexOf(pWord);
	while(vIdx >= 0) {
		var vEnd = vIdx+pWord.length;
		if( (vIdx == 0 || pString.charCodeAt(vIdx-1) < 48) && (vEnd==pString.length || pString.charCodeAt(vEnd) < 48) ) return true;
		vIdx = pString.indexOf(pWord, vEnd);
	}
	return false;
}

/**
 * pre-compile un path
 */
scPaLib.compilePath = function(pPath){
	var vPath = pPath.split("/");
	for(var i in vPath) {
		var vPart = vPath[i] = new String(vPath[i]);
		var vAxisCd = vPart.substring(0, 3);
		vPart.axis = this.xAxis[vAxisCd];
		if(vAxisCd == "ide") {
			if(vPart.length>4) vPart.filterId = vPart.substring(4);
		} else {
			if(vPart.length>4) vPart.filter = this.compileFilter(vPart.substring(4));
		}
	}
	vPath.scPath = true;
	return vPath;
}

/**
 * pre-compile un filtre
 */
scPaLib.compileFilter = function(pFilter){
	var vFilter = pFilter.split("|");
	if(vFilter.length>1){
		for(var i = 0; i < vFilter.length; i++) {
			vFilter[i] = vFilter[i].split(".");
			//1er filtre sur le nodeName
			if(vFilter[i].length>0 && vFilter[i][0].length>0) vFilter[i][0] = new RegExp("^(.*:)?"+vFilter[i][0]+"$", "i");
			vFilter.scFilter = 2;
		}
	}else{
		vFilter = pFilter.split(".");
		//1er filtre sur le nodeName
		if(vFilter.length>0 && vFilter[0].length>0) vFilter[0] = new RegExp("^(.*:)?"+vFilter[0]+"$", "i");
		vFilter.scFilter = 1;
	}
	return vFilter;
}


/* ############### Interne ############# */

scPaLib.xAxis= {
	anc:{
		first:function(pFrom){return pFrom.parentNode;}
	},
    par:{
		next:function (){return null;}
	},
    chi:{
		first:function(pFrom){return pFrom.firstChild;},
		next:function(pFrom){return pFrom.nextSibling;}
	},
    chl:{
		first:function(pFrom){return pFrom.lastChild;}
	},
	nsi:{
	},
	psi:{
		first:function(pFrom){return pFrom.previousSibling;}
	},
    cde:{
		first:function(pFrom){return pFrom;},
		next:function(pFrom, pOri){
			if(pFrom.hasChildNodes()) return pFrom.firstChild;
			if(pFrom.nextSibling) return pFrom.nextSibling;
			while(pFrom != pOri && (pFrom = pFrom.parentNode) != null && pFrom != pOri) {
				var vNext = pFrom.nextSibling;
				if(vNext) return vNext;
			}
			return null;
		}
	},
    des:{
	},
	bod:{
		first:function(pFrom){
			if(pFrom.ownerDocument) return pFrom.ownerDocument.body;
			var vDoc = pFrom;
			while(vDoc.parentNode) vDoc = vDoc.parentNode;
			return vDoc.body;
		}
	},
	ide:{
 		first:function(pFrom, pPart){return pFrom.ownerDocument ? pFrom.ownerDocument.getElementById(pPart.filterId) : pFrom.getElementById(pPart.filterId);}
	},
	init: function(){
		this.anc.next = this.anc.first;
		this.des.first = this.chi.first;
		this.des.next = this.cde.next;
		this.par.first = this.anc.first;
		this.nsi.first = this.chi.next;
		this.nsi.next = this.chi.next;
		this.psi.next = this.psi.first;
		this.chl.next = this.psi.next;
		this.bod.next = this.par.next;
		this.ide.next = this.par.next;
	}
}
scPaLib.xAxis.init();

scPaLib.xFindNode= function(pPath, pFrom, pOffset) {
	var vPart = pPath[pOffset++];
	var vCurr = vPart.axis.first(pFrom, vPart);
	while(vCurr) {
		if( ! vPart.filter || this.checkNode(vPart.filter, vCurr)) {
			if(pOffset < pPath.length) {
				var vRes = this.xFindNode(pPath, vCurr, pOffset);
				if(vRes) return vRes;
			} else {
				return vCurr;
			} 
		}
		vCurr = vPart.axis.next(vCurr, pFrom);
	}
	return null;
}

scPaLib.xFindNodes = function(pPath, pFrom, pOffset, pArrayRes) {
	var vPart = pPath[pOffset++];
	var vCurr = vPart.axis.first(pFrom, vPart);
	while(vCurr) {
		if( ! vPart.filter || this.checkNode(vPart.filter, vCurr)) {
			if(pOffset < pPath.length) {
				this.xFindNodes(pPath, vCurr, pOffset, pArrayRes);
			} else {
				pArrayRes[pArrayRes.length] = vCurr;
			} 
		}
		vCurr = vPart.axis.next(vCurr, pFrom);
	}
}

scPaLib.xCheckNode = function(pFilter, pNode){
	var vLen = pFilter.length;
	if(vLen==0) return true;
	var vStep0 = pFilter[0];
	if(vStep0.test && ! vStep0.test(pNode.nodeName)) return false;
	if(vLen>1) {
		var vClasses = pNode.nodeType==1 ? pNode.className : null;
		if( ! vClasses) return false;
		for(var i = 1; i < vLen; i++) if( ! this.containWord(vClasses, pFilter[i])) return false;
	}
	return true;
}


