/**
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is kelis.fr code.
 *
 * The Initial Developer of the Original Code is 
 * samuel.monsarrat@kelis.fr
 *
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
 * instead of those above. If you wish to allow use of your version of this file
 * only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
 * others to use your version of this file under the terms of the MPL, indicate
 * your decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
 * delete the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */

/**
 * OPA manager.
 */
var opaMgr = {
	fNodeId: null,   // Unique node ID - MUST be defined before onLoad() is executed.
	fNodeType: null, // Node type (exp, heavy etc.) - MUST be defined before onLoad() is executed.
	fSessKey: "",    // assmnt session.
	fIdxs: [],       // array of index objects
	fNxtNodes: [],   // array of next nodes (DOM)
	fNxtCnds: [],    // array of ARC next condition objects (see opaMgr.addArcCnd())
	fHistMgr : null,
	fPathArcRoot : "ide:tplArc",
	fPathArcNodes : "des:.arc_li",
	fPathBtnNext : "ide:nextBtn",
	fPathBtnBack : "ide:backBtn",

/**
 * opaMgr.onLoad() - sc api on load init function.
 */
	onLoad : function(){
		try{
			if (!this.fNodeId || !this.fNodeType) throw new Error("OPA manager not correctly initialized. missing node id ot node type.");
			scCoLib.util.log("opaMgr.onLoad - node ID="+this.fNodeId);

			// history manager
			this.fHistMgr = new histMgr;
			this.fHistMgr.init();

			// node specific init.
			switch (this.fNodeType){
				case "case": {
					return;
				}
				case "assmnt": {
					if(!this.isNodeVisited(this.fNodeId)) scServices.assmntMgr.setCompletionStatus(this.fNodeId, this.fSessKey, "attempt");
					break;
				}
				case "exp":
				case "heavy": {
					if(!this.isNodeVisited(this.fNodeId)) scServices.assmntMgr.setCompletionStatus(this.fNodeId, this.fSessKey, "attempt");
					scServices.assmntMgr.setPts(this.fNodeId, this.fSessKey, 0, 1, 1);
					break;
				}
			}

			// Manage nav buttons
			scPaLib.findNode(this.fPathBtnNext).style.visibility = (this.fHistMgr.hasNext() ? "" : "hidden");
			scPaLib.findNode(this.fPathBtnBack).style.visibility = (this.fHistMgr.hasBack() ? "" : "hidden");

			// Manage Next links
			this.fArcRoot = scPaLib.findNode(this.fPathArcRoot);
			this.fNxtNodes = scPaLib.findNodes(this.fPathArcNodes,this.fArcRoot);
			this.initArc();
			this.fArcRoot.style.visibility = "";
			
		} catch(e){scCoLib.util.logError("ERROR opaMgr.onLoad",e)}
	},

/**
 * opaMgr.isNodeVisited()
 * @param pNodeId : Node ID
 * @returns true if the node has been visited
 */
	isNodeVisited : function(pNodeId){
		return (scServices.assmntMgr.getCompletionStatus(pNodeId, this.fSessKey)!="notAttempt");
	},

/**
 * opaMgr.addArcCnd() - Add an ARC condition to the list that is processed on load. 
 * The order in which the conditions are added defines the corresponding next link that they control
 * condition objects are of the form: {id:"order id",pl:"payload object",op:"operation ex: %>",vl:"compare value"}
 * payload objects are of the form:   {id:"object unique id",tp:"type (n=node, u=userIndex, a=autoIndex)"}
 * @param pCnd : conditon object
 */
	addArcCnd : function(pCnd){
		this.fNxtCnds[this.fNxtCnds.length] = pCnd;
	},

/**
 * opaMgr.initArc() - generic ARC init function.
 * Empty by default. Can be replaced with one of the initArc* functions on init.
 */
	initArc : function(){
	},

/**
 * opaMgr.initArcChoose() - Choose ARC init function.
 * processes the ARC conditions to determine which next node link to display.
 */
	initArcChoose : function(){
		if (this.fNxtCnds.length>0){
			var vNxtFound = false;
			for (var i in this.fNxtCnds) {
				var vCnd = this.fNxtCnds[i];
				var vDisplayNxt = false;
				if (vCnd.op) { // this is a standard cond
					var vPl = null;
					var vOp;
					var vVl = this.xToFloat(vCnd.vl);
					var vIsNode = vCnd.pl.tp == "n";
					if (vCnd.op.charAt(0) == "%") { //this is a % cond
						vOp = vCnd.op.substring(1);
						if (vIsNode && this.isNodeVisited(vCnd.pl.id)){
							var vMin = scServices.assmntMgr.getMinPts(vCnd.pl.id, this.fSessKey);
							var vScore = scServices.assmntMgr.getScorePts(vCnd.pl.id, this.fSessKey);
							var vMax = scServices.assmntMgr.getMaxPts(vCnd.pl.id, this.fSessKey);
							vPl = (vMax==vMin ? (vScore<=0 ? 0 : 100) : (vScore-vMin) / (vMax-vMin) * 100);
						} else if (!vIsNode){
							alert("ERREUR - condition incompatible : Une étape conditionné utilise un indice et un opérateur à pourcentage.")
						}
					} else { // this is a non % cond
						vOp = this.fNxtCnds[i].op;
						if (vIsNode){
							if (this.isNodeVisited(vCnd.pl.id)) vPl = scServices.assmntMgr.getScorePts(vCnd.pl.id, this.fSessKey);
						} else {
							vPl = this.getIndexVal(vCnd.pl.id,vCnd.pl.tp);
						}
					}
					vPl = this.xToFloat(vPl || 0);
					switch (vOp){
						case ">": {
							vDisplayNxt = vPl > vVl; break;
						}
						case ">=": {
							vDisplayNxt = vPl >= vVl; break;
						}
						case "=": {
							vDisplayNxt = vPl == vVl; break;
						}
						case "<": {
							vDisplayNxt = vPl < vVl; break;
						}
						case "<=": {
							vDisplayNxt = vPl <= vVl; break;
						}
					}
					vNxtFound = (vDisplayNxt ? true : vNxtFound);
				} else vDisplayNxt =  !vNxtFound; // must be the default step
				this.fNxtNodes[i].style.display = (vDisplayNxt? "" : "none");
			}
		}
	},

/**
 * opaMgr.initIndex() - Init an auto index.
 */
	initIndex : function(pId,pMode){
		this.fIdxs[pId] = new Object();
		this.fIdxs[pId].md = pMode;
		this.fIdxs[pId].cl = [];
	},

/**
 * opaMgr.addIndexCalc() - Add an auto index calculaton.
 * @param pId : index id
 * @param pInput : input value
 * @param pCalc : calculation object
 * calculation objects are of the form: {op:"operation",vl:"value",mx:"max limit",mn:"min limit"} 
 */
	addIndexCalc : function(pId,pInput,pCalc){
		var vVl = this.xToFloat(pInput);
		if (pCalc){
			for (var i in pCalc) {
				var vCalc = pCalc[i];
				switch (vCalc.op){
					case "r": { //rate
						vVl = vVl * this.xToFloat(vCalc.vl); break;
					}
					case "t": { //translate
						vVl += this.xToFloat(vCalc.vl); break;
					}
					case "m": { //map
						vVl = (vVl>this.xToFloat(vCalc.mx) && vCalc.mx.length>0 ? this.xToFloat(vCalc.vl) : (vVl<this.xToFloat(vCalc.mn) && vCalc.mn.length>0 ? this.xToFloat(vCalc.vl) : vVl)); break;
					}
				}
			}
		}
		this.fIdxs[pId].cl[this.fIdxs[pId].cl.length] = vVl;
	},

/**
 * opaMgr.getIndexVal() - return an index value.
 * @param pId : index id
 * @param pType : index type (auto or user)
 * @returns index value
 */
	getIndexVal : function(pId,pType){
		switch (pType){
			case "a": {
				var vVl = 0;
				for (var i in this.fIdxs[pId].cl) {
					vVl += this.fIdxs[pId].cl[i];
				}
				if (this.fIdxs[pId].md == "average") vVl = vVl / this.fIdxs[pId].cl.length;
				return vVl;
				break;
			}
			default: {
				return (scServices.assmntMgr.getScorePts(pId, this.fSessKey)||"");
			}
		}
	},

/**
 * opaMgr.setUserIndex() - set an index value.
 * @param pId : index id
 * @param pVal : index value
 */
	setIndexVal : function(pId, pVal){
		scServices.assmntMgr.setCompletionStatus(pId, this.fSessKey, "attempt");
		scServices.assmntMgr.setPts(pId, this.fSessKey, pVal, pVal, pVal);
	},

/**
 * opaMgr.setUserIndex() - set a user index.
 * @param pId : index id
 * @param pInput : input DOM node
 */
	setUserIndex : function(pId, pInput){
		if (pInput.value.search("^[0-9.]*$") != -1){
			this.setIndexVal(pId,pInput.value);
		} else {
			alert("Attention seul les valeurs numériques sont acceptés.");
			pInput.value = pInput.fVal;
		}
	},

/**
 * opaMgr.next() - move to next page in the history.
 */
	next : function(){
		if (this.fHistMgr){
			window.location.replace(this.fHistMgr.next());
		}
	},

/**
 * opaMgr.back() - move to next page in the history.
 */
	back : function(){
		if (this.fHistMgr){
			window.location.replace(this.fHistMgr.back());
		}
	},

/**
 * opaMgr.xToFloat() - parseFloat wrapper.
 * @param pX : input
 * @returns a float representation or 0 if input is NaN.
 */
	xToFloat : function(pX){
		var vY;
		return isNaN(vY = parseFloat(pX))? 0 : vY;
	},

	loadSortKey : "O"
}
scOnLoads[scOnLoads.length] = opaMgr;

